<?php

namespace Axepta\Payment;

/**
 * Class Paywall
 *
 * @package Axepta\Payment
 */
class Paywall
{
	const ROUTE_PAY = '/payment';

	/**
	 * @var array
	 */
	public static $paywallUrlList = [
		Util::ENVIRONMENT_PRODUCTION => 'https://paywall.axepta.pl',
		Util::ENVIRONMENT_SANDBOX    => 'https://paywall.sandbox.axepta.pl',
	];
	/**
	 * @var string
	 */
	private $serviceKey;

	/**
	 * @var string
	 */
	private $serviceId;

	/**
	 * @var string
	 */
	private $merchantId;

	/**
	 * @var string
	 */
	private $environment;

	/**
	 * Payment constructor.
	 *
	 * @param string $environment
	 * @param string $serviceKey
	 * @param string $serviceId
	 * @param string $merchantId
	 */
	public function __construct($environment, $serviceKey, $serviceId, $merchantId)
	{
		$this->environment = $environment;
		$this->serviceKey = trim($serviceKey);
		$this->serviceId = trim($serviceId);
		$this->merchantId = trim($merchantId);
	}

	/**
	 * @param int    $amount
	 * @param string $currency
	 * @param string $orderId
	 * @param string $customerFirstName
	 * @param string $customerLastName
	 * @param string $customerEmail
	 * @param string $customerPhone
	 * @param string $successReturnUrl
	 * @param string $failureReturnUrl
	 * @param string $returnUrl
	 * @param string $creditType
	 * @param string $cartData
	 * @param string $version
	 *
	 * @return array
	 */
	public static function prepareData(
		$amount,
        $currency,
        $orderId,
        $customerFirstName,
        $customerLastName,
		$customerEmail,
        $customerPhone = '',
        $successReturnUrl = '',
        $failureReturnUrl = '',
        $returnUrl = '',
        $creditType = '',
        $cartData = '',
        $version = ''
	) {
		$data = [];
		$data['amount'] = $amount;
		$data['currency'] = $currency;
		$data['orderId'] = $orderId;
		$data['customer[firstName]'] = $customerFirstName;
		$data['customer[lastName]'] = $customerLastName;
		$data['customer[email]'] = $customerEmail;

		if($customerPhone) {
			$data['customer[phone]'] = $customerPhone;
		}

		if($successReturnUrl) {
			$data['successReturnUrl'] = $successReturnUrl;
		}

		if($failureReturnUrl){
			$data['failureReturnUrl'] = $failureReturnUrl;
		}

		if($returnUrl){
			$data['returnUrl'] = $returnUrl;
		}

		if($creditType) {
			$data['creditType'] = $creditType;
		}

		if($cartData) {
			$data['cartData'] = $cartData;
		}

		if($version) {
			$data['version'] = $version;
		}

		return $data;
	}

	/**
	 * Creates full form with order data.
	 *
	 * @param array  $orderData
	 * @param string $submitValue
	 * @param string $submitClass
	 * @param string $submitStyle
	 *
	 * @return string
	 */
	public function buildOrderForm($orderData, $submitValue = '', $submitClass = '', $submitStyle = '')
	{
		return Util::createOrderForm(
			$this->prepareOrderData($orderData),
			$this->getPaywallPaymentUrl($this->environment),
			Util::METHOD_REQUEST_POST,
			$submitValue,
			$submitClass,
			$submitStyle
		);
	}

	/**
	 * Simple function that adds signature
	 *
	 * @param array  $order
	 * @param string $hashMethod
	 *
	 * @return array
	 */
	private function prepareOrderData($order, $hashMethod = 'sha256')
	{

		$order['serviceId'] = $this->serviceId;
		$order['merchantId'] = $this->merchantId;
		$order['signature'] = $this->createSignature($order, $this->serviceKey, $hashMethod);

		return $order;
	}

	/**
	 * @param array  $orderData
	 * @param string $serviceKey
	 * @param string $hashMethod
	 *
	 * @return string|bool
	 */
	private function createSignature($orderData, $serviceKey, $hashMethod = 'sha256')
	{

		$isHashMethodSupported = Util::getHashMethod($hashMethod);

		if(!$isHashMethodSupported
			|| !is_array($orderData)) {
			return false;
		}

		return Util::hashSignature($hashMethod, self::prepareHashData($orderData), $serviceKey) . ';' . $hashMethod;
	}

	/**
	 * @param array  $data
	 * @param string $prefix
	 *
	 * @return string
	 */
	public static function prepareHashData($data, $prefix = '')
	{
		ksort($data);

		$hashData = [];
		foreach($data as $key => $value) {

			if($prefix) {
				$key = $prefix . '[' . $key . ']';
			}
			if(is_array($value)) {
				$hashData[] = self::prepareHashData($value, $key);
			} else {
				$hashData[] = $key . '=' . $value;
			}
		}

		return implode('&', $hashData);
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	private function getPaywallPaymentUrl($environment)
	{

		$baseUrl = self::getPaywallUrl($environment);

		if($baseUrl) {
			return $baseUrl . self::ROUTE_PAY;
		}

		return '';
	}

	/**
	 * @param string $environment
	 *
	 * @return string
	 */
	public static function getPaywallUrl($environment)
	{

		if(isset(self::$paywallUrlList[$environment])) {
			return self::$paywallUrlList[$environment];
		}

		return '';
	}
}
